/* splib.c - biblioteca de funcoes */

#include <stdio.h>
#include <fcntl.h>
#include "spool.h"

/*  Resumo das Funcoes. 
 
  Tabela de Tipos de Impressoras: 

    inclui_tipo (char * tipo da impressora)
    altera_tipo (char * tipo da impressora)
    lista_tipo (char * tipo da impressora)
    lista_todos_tipo ()
    remove_tipo (char * tipo da impressora)
    disp_uso_tipo ()
    pesq_tipo (char * tipo da impressora)
      retorna OK se existe a impressora indicada,
      retorna ERRO caso contrario. 
    le_tipo (char *tipo, char *buffer)
      recebe o tipo a ser lido
      retorna o buffer com as informacoes do registro 
      retorna OK, ERRO

  Tabela de Impressoras do Sistema:

    inclui_imp (char * impressora do sistema)
    altera_imp (char * impressora do sistema)
    lista_imp (char * impressora do sistema)
    lista_todos_imp ()
    remove_imp (char * impressora do sistema)
    disp_uso_imp ()
    le_imp (char * impressora, char * buffer)
    grava_imp (char * impressora, char * buffer)

  Tabela de Formularios do Sistema:

    inclui_form (char * formulario)
    altera_form (char * formulario)
    lista_form (char * formularioi)
    lista_todos_form ()
    remove_form (char * formulario)
    disp_uso_form ()
    le_form (char * formulario, char * buffer)
      recebe o nome do formulario
      retorna o buffer com os dados do registro
      retorna OK, ERRO

  Tabela de Sequencia de Impressao

    le_sequencia ()
      retorna um inteiro contento o numero da ultima requisicao.
    grava_sequencia (int numero de sequencia)
      recebe um inteiro indicando qual foi a ultima requisicao.

  Tabela de pid do Spool

    le_pid ()
      retorna o numero do pid correspondente do spool ativo.
    grava_pid (int numero do pid)
      recebe um inteiro com o pid do novo spool ativo.

  Fila de Impressao

    acrescenta_fila (char * buffer)
      recebe o buffer ja pre-formatado e o coloca no final da fila de impressao.
    le_req (char * numero de requisicao, char * buffer)
      recebe um string contendo o numero da requisicao,
      retorna a requisicao no buffer cedido.
    grava_req (char * numero de requisicao, char * buffer)
      recebe um string com o numero da requisicao para ser sobreposta,
      recebe o buffer do registro ja pre-formatado.
    pega_prox_req_ativa (FILE * arquivo, char * buffer)
      recebe um ponteiro para arquivo e o le de acordo com o ponteiro corrente,
      retorna o buffer indicado.
    
  Manipulacao de arquivo

    abre (int * arquivo, char * nome arquivo, char * modo)
      recebe um string com o nome do arquivo para ser aberto,
      recebe o modo de abertura do arquivo: "r", "w", "r+w", "a", "create"
      retorna um inteiro correspondente ao arquivo aberto.
    le_seq (int arquivo, char * buffer, int tamanho do registro)
      recebe um inteiro correspondente ao arquivo
      recebe um inteiro com o tamanho do registro a ser lido
      retorna o buffer preenchido com as informacoes do registro apontado pelo 
      ponteiro corrente do arquivo.
    pesq_req (int arquivo, char * buffer, int tamanho do reg, char * chave)
      recebe um inteiro correspondente ao arquivo
      recebe um inteiro com o tamanho do registro
      recebe ponteiro para string com a chave de busca
      retorna um ponteiro para o buffer preenchido.
    regrava_reg (int arquivo, char * buffer, int tamanho do registro)
      recebe um inteiro com o arquivo
      recebe um inteiro com o tamanho do registro
      recebe o buffer sem pre-formatacao. 
    pega_cpo (char * origem, int numero do campo, char * destino)
      recebe ponteiro para string com a origem (buffer)
      recebe um inteiro para o numero do campo pedido
      retorna o campo no destino.
    poe_cpo (char *destino, int numero do campo, char * origem)
      mesmo procedimento do pega_cpo so que coloca a origem no buffer.
    strtoken (char * buffer, char * token)
      modulo que substitui a strtok da libc pois a mesma nao funciona per-  
      feitamente quando se tem um campo nulo entre dois tokens. Ex: 99::99 */

/* Tipo de impressoras */

inclui_tipo (tipo)
char *tipo;
{
  int fdtipo;
  char buftipo [TAMTIP];
  char comp [30], inic [30], fim [30], pag [30],
       sexto [30], oito [30], carta [30], normal [30];
  int resp;

  do {
    printf ("confirma a inclusao da impressora tipo '%s' (s/n).:", tipo);
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO)
    return (ERRO);
  abre (&fdtipo, TABTIPO, "a");
  printf ("\nInforme as caracteristicas da impressora tipo '%s'\n\n", tipo);
  printf ("   sequencia de controle p/ inicio de impressao..: ");
  gets (inic);
  printf ("   sequencia de controle p/ fim de impressao.....: ");
  gets (fim);
  printf ("   sequencia de controle p/ impressao normal.....: ");
  gets (normal);
  printf ("   sequencia de controle p/ impressao comprimida.: ");
  gets (comp);
  printf ("   sequencia de controle p/ tamanho da pagina....: ");
  gets (pag);
  printf ("   sequencia de controle p/ espacamento de 1/6...: ");
  gets (sexto);
  printf ("   sequencia de controle p/ espacamento de 1/8...: ");
  gets (oito);
  printf ("   sequencia de controle p/ qualidade carta......: ");
  gets (carta);
  do {
    printf ("\nconfirma a gravacao (s/n).:");
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO) {
    close (fdtipo);
    return (ERRO);
  }
  sprintf (buftipo, "%s:%s:%s:%s:%s:%s:%s:%s:%s:", tipo, inic, fim, normal,
           comp, pag, sexto, oito, carta);
  grava_reg (fdtipo, buftipo, TAMTIP);
  close (fdtipo);
  return (OK); 
}

altera_tipo (tipo)
char *tipo;
{
  int fdtipo;
  char buftipo [TAMTIP];
  char comp [30], inic [30], fim [30], pag [30],
       sexto [30], oito [30], carta [30], normal [30];
  int resp;

  abre (&fdtipo, TABTIPO, "r+w");
  if (pesq_reg (fdtipo, buftipo, TAMTIP, tipo) != OK) {
    printf ("sptipo: impressora tipo '%s' nao existente\n", tipo);
    close (fdtipo);
    return (ERRO);
  }
  pega_cpo (buftipo, TIPINI, inic);
  pega_cpo (buftipo, TIPFIM, fim);
  pega_cpo (buftipo, TIPNOR, normal);
  pega_cpo (buftipo, TIPCOM, comp);
  pega_cpo (buftipo, TIPPAG, pag);
  pega_cpo (buftipo, TIPSEX, sexto);
  pega_cpo (buftipo, TIPOIT, oito);
  pega_cpo (buftipo, TIPCAR, carta);
  printf ("\n\nInforme as caracteristicas da impressora tipo '%s'\n\n", tipo);
  printf ("   sequencia de controle p/ inicio de impressao.(%s).: ", inic);
  gets (inic);
  printf ("   sequencia de controle p/ fim de impressao....(%s).: ", fim);
  gets (fim);
  printf ("   sequencia de controle p/ impressao normal....(%s).: ", normal);
  gets (normal);
  printf ("   sequencia de controle p/ impressao comprimida(%s).: ", comp);
  gets (comp);
  printf ("   sequencia de controle p/ tamanho da pagina...(%s).: ", pag);
  gets (pag);
  printf ("   sequencia de controle p/ espacamento de 1/6..(%s).: ", sexto);
  gets (sexto);
  printf ("   sequencia de controle p/ espacamento de 1/8..(%s).: ", oito);
  gets (oito);
  printf ("   sequencia de controle p/ qualidade carta.....(%s).: ", carta);
  gets (carta);
  do {
    printf ("\nconfirma a gravacao (s/n).:");
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO) {
    close (fdtipo);
    return (ERRO);
  }
  if (strlen (inic) != 0)
    poe_cpo (buftipo, TIPINI, inic);
  if (strlen (fim) != 0)
    poe_cpo (buftipo, TIPFIM, fim);
  if (strlen (normal) != 0)
    poe_cpo (buftipo, TIPNOR, normal);
  if (strlen (comp) != 0)
    poe_cpo (buftipo, TIPCOM, comp);
  if (strlen (pag) != 0)
    poe_cpo (buftipo, TIPPAG, pag);
  if (strlen (sexto) != 0)
    poe_cpo (buftipo, TIPSEX, sexto);
  if (strlen (oito) != 0)
    poe_cpo (buftipo, TIPOIT, oito);
  if (strlen (carta) != 0)
    poe_cpo (buftipo, TIPCAR, carta);
  regrava_reg (fdtipo, buftipo, TAMTIP);
  close (fdtipo);
  return (OK); 
}

lista_tipo (tipo)
char *tipo;
{
  int  fdtipo;
  char buftipo [TAMTIP];
  char comp [30], inic [30], fim [30], pag [30],
       sexto [30], oito [30], carta [30], normal [30];

  abre (&fdtipo, TABTIPO, "r");
  if (pesq_reg (fdtipo, buftipo, TAMTIP, tipo) != OK) {
    printf ("sptipo: impressora tipo '%s' nao existente\n", tipo);
    close (fdtipo);
    return (ERRO);
  }
  pega_cpo (buftipo, TIPINI, inic);
  pega_cpo (buftipo, TIPFIM, fim);
  pega_cpo (buftipo, TIPNOR, normal);
  pega_cpo (buftipo, TIPCOM, comp);
  pega_cpo (buftipo, TIPPAG, pag);
  pega_cpo (buftipo, TIPSEX, sexto);
  pega_cpo (buftipo, TIPOIT, oito);
  pega_cpo (buftipo, TIPCAR, carta);
  printf ("\nCaracteristicas da impressora tipo '%s'\n\n", tipo);
  printf ("   sequencia de controle p/ inicio de impressao..: %s\n", inic);
  printf ("   sequencia de controle p/ fim de impressao.....: %s\n", fim);
  printf ("   sequencia de controle p/ impressao normal.....: %s\n", normal);
  printf ("   sequencia de controle p/ impressao comprimida.: %s\n", comp);
  printf ("   sequencia de controle p/ tamanho da pagina....: %s\n", pag);
  printf ("   sequencia de controle p/ espacamento de 1/6...: %s\n", sexto);
  printf ("   sequencia de controle p/ espacamento de 1/8...: %s\n", oito);
  printf ("   sequencia de controle p/ qualidade carta......: %s\n", carta);
  close (fdtipo);
}

lista_todos_tipo ()
{
  int  fdtipo;
  char buftipo [TAMTIP];
  char tipo [30], comp [30], inic [30], fim [30], pag [30],
       sexto [30], oito [30], carta [30], normal [30];

  abre (&fdtipo, TABTIPO, "r");
  printf ("TIPO                INICIO        FINAL");
  printf ("         NORMAL        COMPAC.\n");
  printf ("                    TAM.PAG       SEXTO");
  printf ("         OITAVO        CARTA\n");
  while (le_seq (fdtipo, buftipo, TAMTIP) == OK) {
    pega_cpo (buftipo, TIPTIP, tipo);
    pega_cpo (buftipo, TIPINI, inic);
    pega_cpo (buftipo, TIPFIM, fim);
    pega_cpo (buftipo, TIPNOR, normal);
    pega_cpo (buftipo, TIPCOM, comp);
    pega_cpo (buftipo, TIPPAG, pag);
    pega_cpo (buftipo, TIPSEX, sexto);
    pega_cpo (buftipo, TIPOIT, oito);
    pega_cpo (buftipo, TIPCAR, carta);
    printf ("%-19.19s %-13.13s %-13.13s %-13.13s %-13.13s\n",
            tipo, inic, fim, normal, comp);
    printf ("                    %-13.13s %-13.13s %-13.13s %-13.13s\n",
            pag, sexto, oito, carta);
  }
  close (fdtipo);
}

remove_tipo (tipo)
char *tipo;
{
  char cmd [512],  auxtipo [50], buftipo [TAMTIP];
  int  resp, fdtipo, fdntipo;
  char comp [30], inic [30], fim [30], pag [30],
       sexto [30], oito [30], carta [30], normal [30];

  abre (&fdtipo, TABTIPO, "r");
  if (pesq_reg (fdtipo, buftipo, TAMTIP, tipo) != OK) {
    printf ("sptipo: impressora tipo '%s' nao existente\n", tipo);
    close (fdtipo);
    return (ERRO);
  }
  lseek (fdtipo, 0L, 0);
  abre (&fdntipo, TABNTIPO, "create");
  while (le_seq (fdtipo, buftipo, TAMTIP) == OK) {
    pega_cpo (buftipo, TIPTIP, auxtipo);
    if (strcmp (tipo, auxtipo) != OK)
      grava_reg (fdntipo, buftipo, TAMTIP);
  }
  close (fdtipo);
  close (fdntipo);

  strcpy (cmd, "cp ");
  strcat (cmd, TABNTIPO);
  strcat (cmd, " ");
  strcat (cmd, TABTIPO);
  system (cmd);
  return (OK); 
}

disp_uso_tipo ()
{
  printf ("usage : sptipo [-l] [-r] [tipo]\n");
  exit (0);
}

pesq_tipo (tipo)
char *tipo;
{
  int fdtipo;
  char buftipo [TAMTIP]; 

  abre (&fdtipo, TABTIPO, "r");
  while (le_seq (fdtipo, buftipo, TAMTIP) == OK)
  if (strncmp (tipo, buftipo, strlen (tipo)) == OK) {
    close (fdtipo);
    return (OK);
  }
  close (fdtipo);
  return (ERRO);
}

le_tipo (tipo, buffer)
char *tipo, *buffer;
{
  int fd;

  abre (&fd, TABTIPO, "r");
  if (pesq_reg (fd, buffer, TAMTIP, tipo) != OK) {
    printf ("spadm: Tipo %s nao cadastrada\n", tipo);
    close (fd);
    return (ERRO);
  }
  close (fd);
  return (OK);
}  

/* impressoras do sistema */

inclui_imp (imp)
char *imp;
{
  int fdimp;
  char bufimp [TAMIMP];
  char dev [30], tipo [30];
  int  resp;
  
  do {
    printf ("confirma a inclusao da impressora '%s' (s/n).:", imp);
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO)
    return (ERRO);
  abre (&fdimp, TABIMP, "a");
  printf ("\nInforme as caracteristicas da impressora '%s'\n\n", imp);
  printf ("   Nome do device (/dev/xxx)..: ");
  gets (dev);
  do {
    printf ("   Tipo da impressora.........: ");
  gets (tipo);
  } while (pesq_tipo (tipo) != OK);
  do {
    printf ("\nconfirma a gravacao (s/n).:");
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO) {
    close (fdimp);
    return (ERRO);
  }
  sprintf (bufimp, "%s:%s:%s:%s:%s:%s:%s:%s:%s:",
           imp, dev, tipo, DESATIVADA, ESPACO, NORMAL, ESPACO, ESPACO, "0");
  grava_reg (fdimp, bufimp, TAMIMP);
  close (fdimp);
  return (OK);
}
 
altera_imp (imp)
char *imp;
{
  int fdimp;
  char bufimp [TAMIMP];
  char dev [30], tipo[30], auxtipo [30]; 
  int  resp;

  abre (&fdimp, TABIMP, "r+w");
  if (pesq_reg (fdimp, bufimp, TAMIMP, imp) != OK) {
    printf ("spimp: impressora '%s' nao existente\n", imp);
    close (fdimp);
    return (ERRO);
  }
  pega_cpo (bufimp, IMPDEV, dev);
  pega_cpo (bufimp, IMPTIP, tipo);
  printf ("\nInforme as caracteristicas da impressora '%s'\n\n", imp);
  printf ("   Nome do device .......(%s).: ", dev);
  gets (dev);
  strcpy (auxtipo, tipo);
  do {
    printf ("   Tipo da impressora....(%s).: ", auxtipo);
    gets (tipo);
  } while (pesq_tipo (tipo) != OK && strlen (tipo) != 0);
  do {
     printf ("\nconfirma a gravacao (s/n).:");
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO) {
    close (fdimp);
    return (ERRO);
  }
  if (strlen (dev) != 0)
    poe_cpo (bufimp, IMPDEV, dev);
  if (strlen (tipo) != 0)
    poe_cpo (bufimp, IMPTIP, tipo);
  regrava_reg (fdimp, bufimp, TAMIMP);
  close (fdimp);
  return (OK); 
}

lista_imp (imp)
char *imp;
{
  int fdimp;
  char bufimp [TAMIMP];
  char nome [60], device [60], tipo [60], estado [60],
       ultimo [60], forms [60], pagina [10];

  abre (&fdimp, TABIMP, "r");
  if (pesq_reg (fdimp, bufimp, TAMIMP, imp) != OK) {
    printf ("spimp: impressora '%s' nao existente\n", imp);
    close (fdimp);
    return (ERRO);
  }
  pega_cpo (bufimp, IMPIMP, nome); 
  pega_cpo (bufimp, IMPDEV, device); 
  pega_cpo (bufimp, IMPTIP, tipo); 
  pega_cpo (bufimp, IMPEST, estado); 
  pega_cpo (bufimp, IMPFOR, forms); 
  pega_cpo (bufimp, IMPPAG, pagina);
  printf ("\nCaracteristicas da impressora '%s'\n\n", imp);
  printf ("   Impressora..........: %s\n", nome);
  printf ("   Nome do device......: %s\n", device);
  printf ("   Tipo da impressora..: %s\n", tipo);
  printf ("   Estado da impressora: %s\n", estado);
  printf ("   Formulario corrente.: %s\n", forms);
  printf ("   Pagina corrente.....: %s\n", pagina);
  close (fdimp);
}

lista_todos_imp ()
{
  int fdimp;
  char bufimp [TAMIMP];
  char imp [60], dev [60], tipo[60], estado [60], forms [60], pagina [10];

  abre (&fdimp, TABIMP, "r");
  printf ("IMPRESSORA    DEVICE        TIPO");
  printf ("          ESTADO     FORMULARIO     PAGINA IMPR.\n"); 
  while (le_seq (fdimp, bufimp, TAMIMP) == OK) {
    pega_cpo (bufimp, IMPIMP, imp);
    pega_cpo (bufimp, IMPDEV, dev);
    pega_cpo (bufimp, IMPTIP, tipo);
    pega_cpo (bufimp, IMPEST, estado);
    pega_cpo (bufimp, IMPFOR, forms);
    pega_cpo (bufimp, IMPPAG, pagina);
    printf ("%-13.13s %-13.13s %-13.13s %-13.13s %-15.15s %-15.15s\n",
    imp, dev, tipo, estado, forms, pagina);
  }
  close (fdimp);
}

remove_imp (imp)
char *imp;
{
  int fdimp, fdnimp;
  char bufimp [TAMIMP];
  char cmd [512], auximp [30];

  abre (&fdimp, TABIMP, "r");
  if (pesq_reg (fdimp, bufimp, TAMIMP, imp) != OK) {
    printf ("spimp: impressora '%s' nao existente\n", imp);
    return (ERRO);
  }
  lseek (fdimp, 0L, 0);
  abre (&fdnimp, TABNIMP, "create");
  while (le_seq (fdimp, bufimp, TAMIMP) == OK) {
    pega_cpo (bufimp, IMPIMP, auximp);
    if (strcmp (imp, auximp) != OK)
      grava_reg (fdnimp, bufimp, TAMIMP);
  }
  close (fdimp);
  close (fdnimp);

  strcpy (cmd, "cp ");
  strcat (cmd, TABNIMP);
  strcat (cmd, " ");
  strcat (cmd, TABIMP);
  system (cmd);
  return (OK); 
}

disp_uso_imp ()
{
  printf ("usage : spimp [-l] [-r] [impressora]\n");
  exit (0);
}

le_imp (impressora, buffer)
char *impressora, *buffer;
{
  int fd;

  abre (&fd, TABIMP, "r");
  if (pesq_reg (fd, buffer, TAMIMP, impressora) != OK) {
    printf ("spadm: impressora %s nao cadastrada\n", impressora);
    close (fd);
    return (ERRO);
  }
  close (fd);
  return (OK);
}  

grava_imp (impressora, buffer)
char *impressora, *buffer;
{
  int fd;
  char bufaux [512];
  int i;

  abre (&fd, TABIMP, "r+w");
  if (pesq_reg (fd, bufaux, TAMIMP, impressora) != OK) {
    printf ("erro: a impressora %s nao esta cadastrada\n", impressora);
    close (fd);
    return (ERRO);
  }
  regrava_reg (fd, buffer, TAMIMP);
  close (fd);
  return (OK);
}

/* formularios */

inclui_form (form)
char *form;
{
  int fdform;
  char lin [30], entre[30], comp[30], carta[30];
  char bufform [TAMFOR];
  int resp;

  do {
    printf ("confirma a inclusao do formulario '%s' (s/n).:", form);
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO)
    return (ERRO);
  abre (&fdform, TABFORM, "a");
  printf ("\nInforme as caracteristicas do formulario '%s'\n\n", form);
  printf ("   Tamanho da pagina em linhas...................: ");
  gets (lin);
  do {
    printf ("   Espacamento entre linhas (sexto ou oitavo)....: ");
    gets (entre);
  } while (strcmp (entre, "sexto") && strcmp (entre, "oitavo"));
  do {
    printf ("   Espacamento horizontal (normal ou comprimido).: ");
    gets (comp);
  } while (strcmp (comp, "normal") && strcmp (comp, "comprimido"));
  do {
    printf ("   Qualidade de impressao (padrao ou carta)......: ");
  gets (carta);
  } while (strcmp (carta, "padrao") && strcmp (carta, "carta"));
  do {
    printf ("\nconfirma a gravacao (s/n).:");
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO) {
    close (fdform);
    return (ERRO);
  }
  sprintf (bufform, "%s:%s:%s:%s:%s:\n", form, lin, entre, comp, carta);
  grava_reg (fdform, bufform, TAMFOR);
  close (fdform);
  return (OK); 
}

altera_form (form)
char *form;
{
  int fdform;
  char lin [30], entre [30], comp [30], carta [30],
                 entreant [30], compant [30], cartaant [30];
  char bufform [TAMFOR];
  int resp;

  abre (&fdform, TABFORM, "r+w");
  pesq_reg (fdform, bufform, TAMFOR, form);
  pega_cpo (bufform, FORLIN, lin);
  pega_cpo (bufform, FORLIN, lin);
  pega_cpo (bufform, FORENT, entre);
  pega_cpo (bufform, FORCOM, comp);
  pega_cpo (bufform, FORPAD, carta);
  printf ("\nInforme as caracteristicas do formulario '%s'\n\n", form);
  printf ("   Tamanho da pagina em linhas ..(%s).: ", lin);
  gets (lin);
  strcpy (entreant, entre);
  do {
    printf ("   Espacamento entre linhas .....(%s).: ", entreant);
    gets (entre);
  } while (strcmp (entre, "sexto") && strcmp (entre, "oitavo") &&
           strcmp (entre, ""));
  strcpy (compant, comp);
  do {
    printf ("   Espacamento horizontal .......(%s).: ", compant);
    gets (comp);
  } while (strcmp (comp, "normal") && strcmp (comp, "comprimido") &&
           strcmp (comp, ""));
  strcpy (cartaant, carta);
  do {
    printf ("   Qualidade de impressao .......(%s).: ", cartaant);
    gets (carta);
  } while (strcmp (carta, "padrao") && strcmp (carta, "carta") &&
           strcmp (carta, ""));
  do {
    printf ("\nconfirma a gravacao (s/n).:");
  } while ((resp = pegasn ()) == INVALIDO);
  if (resp == NAO) {
    close (fdform);
    return (ERRO);
  }
  if (strlen (lin) != 0)
    poe_cpo (bufform, FORLIN, lin); 
  if (strlen (entre) != 0)
    poe_cpo (bufform, FORENT, entre); 
  if (strlen (comp) != 0)
    poe_cpo (bufform, FORCOM, comp); 
  if (strlen (carta) != 0)
    poe_cpo (bufform, FORPAD, carta);
  regrava_reg (fdform, bufform, TAMFOR);
  close (fdform);
  return (OK); 
}

lista_form (form)
char *form;
{
  int fdform;
  char lin [30], entre [30], comp [30], carta [30];
  char bufform [TAMFOR];

  abre (&fdform, TABFORM, "r");
  if (pesq_reg (fdform, bufform, TAMFOR, form) != OK) {
    printf ("spform: formulario '%s' nao existente\n", form);
    close (fdform);
    return (ERRO);
  }
  pega_cpo (bufform, FORLIN, lin);
  pega_cpo (bufform, FORENT, entre);
  pega_cpo (bufform, FORCOM, comp);
  pega_cpo (bufform, FORPAD, carta);
  printf ("\nCaracteristicas do formulario '%s'\n\n", form);
  printf ("   Tamanho da pagina em linhas ..: %s\n", lin);
  printf ("   Espacamento entre linhas .....: %s\n", entre);
  printf ("   Espacamento horizontal .......: %s\n", comp);
  printf ("   Qualidade de impressao .......: %s\n", carta);
  close (fdform);
}

lista_todos_form ()
{
  int fdform;
  char form [30], lin [30], entre [30], comp [30], carta [30];
  char bufform [TAMFOR];

  abre (&fdform, TABFORM, "r");
  printf ("FORMULARIO       TAMANHO  ESPACAMENTO  ESPACAMENTO  QUALIDADE\n");
  printf ("                 PAGINA   ENTRE LINHA  HORIZONTAL   IMPRESSAO\n");
  while (le_seq (fdform, bufform, TAMFOR) == OK) {
    pega_cpo (bufform, FORFOR, form);
    pega_cpo (bufform, FORLIN, lin);
    pega_cpo (bufform, FORENT, entre);
    pega_cpo (bufform, FORCOM, comp);
    pega_cpo (bufform, FORPAD, carta);
    printf ("%-16.16s %-8.8s %-12.12s %-12.12s %-12.12s\n",
            form, lin, entre, comp, carta);
  }
  close (fdform);
}

remove_form (form)
char *form;
{
  int fdform, fdnform;
  char lin [30], entre [30], comp [30], carta [30];
  char bufform [TAMFOR];
  char cmd[512], auxform [50];

  abre (&fdform, TABFORM, "r");
  if (pesq_reg (fdform, bufform, TAMFOR, form) != OK) {
    printf ("spform: formulario '%s' nao existente\n", form);
    close (fdform);
    return (ERRO);
  }
  lseek (fdform, 0L, 0);
  abre (&fdnform, TABNFORM, "create");
  while (le_seq (fdform, bufform, TAMFOR) == OK) {
    pega_cpo (bufform, FORFOR, auxform);
    if (strcmp (form, auxform) != OK)
      grava_reg (fdnform, bufform, TAMFOR);
  }
  close (fdform);
  close (fdnform);

  strcpy (cmd, "cp ");
  strcat (cmd, TABNFORM);
  strcat (cmd, " ");
  strcat (cmd, TABFORM);
  system (cmd);
  return (OK); 
}

disp_uso_form ()
{
  printf ("usage : spform [-l] [-r] [formulario]\n");
  exit (0);
}

le_form (form, buffer)
char *form, *buffer;
{
  int fd;

  abre (&fd, TABFORM, "r");
  if (pesq_reg (fd, buffer, TAMFOR, form) != OK) {
    printf ("spadm: Formulario %s nao cadastrada\n", form);
    close (fd);
    return (ERRO);
  }
  close (fd);
  return (OK);
}  

/* Tabela de sequencia */

le_sequencia ()
{
  FILE *fdseq;
  char buffer [TAMSEQ];
  int proxseq;

  if ((fdseq = fopen (TABSEQ, "r+w")) == NULL) {
    printf ("Erro ao abrir o arquivo %s\n", TABSEQ);
    exit (0);
  }
  if (fgets (buffer, TAMSEQ, fdseq) == NULL) {
    printf ("Erro ao ler o arquivo %s\n", TABSEQ);
    exit (0);
  }
  fclose (fdseq);
  return (atoi (buffer));
}

grava_sequencia (sequencia)
int sequencia;
{
  FILE *fdseq;
  char buffer [TAMSEQ];
  int proxseq;

  if ((fdseq = fopen (TABSEQ, "r+w")) == NULL) {
    printf ("Erro ao abrir o arquivo %s\n", TABSEQ);
    exit (0);
  }
  sprintf (buffer, "%d\n", sequencia);
  fseek (fdseq, 0L, 0);
  fputs (buffer, fdseq);
  fclose (fdseq);
}

/* Tabela de pid */

le_pid ()
{
  FILE *fdpid;
  char buffer [TAMPID];

  if ((fdpid = fopen (TABPID, "r+w")) == NULL) {
    printf ("Erro ao abrir o arquivo %s\n", TABPID);
    exit (0);
  }
  if (fgets (buffer, TAMPID, fdpid) == NULL) {
    printf ("Erro: o arquivo %s esta vazio\n", TABPID);
    return (ERRO);
  }
  fclose (fdpid);
  return (atoi (buffer));
}

grava_pid (pid)
int pid;
{
  FILE *fdpid;
  char buffer [TAMPID];

  if ((fdpid = fopen (TABPID, "r+w")) == NULL) {
    printf ("Erro ao abrir o arquivo %s\n", TABPID);
    exit (0);
  }
  sprintf (buffer, "%d\n", pid);
  fseek (fdpid, 0L, 0);
  fputs (buffer, fdpid);
  fclose (fdpid);
}

/* Fila de impressao */

acrescenta_fila (buffer)
char *buffer;
{
  FILE *fdfila;
  
  if ((fdfila = fopen (TABFILA, "a")) == NULL) {
    printf ("Erro ao abrir o arquivo %s\n", TABSEQ);
    exit (0);
  }
  fputs (buffer, fdfila);
  fclose (fdfila);
}

le_req (nreq, buffer)
char *nreq, *buffer;
{
   FILE *fd;

   if ((fd = fopen (TABFILA, "r"))==NULL) {
     printf ("Erro na abertura do arquivo %s\n", TABFILA);
     return (ERRO);
   }
   while (1) {
     if (fgets(buffer, TAMFIL, fd) == NULL) {
       fclose (fd);
       return (ERRO);
     }
     if (strncmp (nreq, buffer, strlen (nreq)) == OK)
       break;
   }
   fclose (fd);
   return (OK);
}  

grava_req (nreq, buffer)
char *nreq, *buffer;
{
  FILE *fd;
  char bufaux [512];
  int i;

  if ((fd = fopen (TABFILA, "r+w"))==NULL) {
    printf ("Erro na abertura do arquivo %s\n", TABFILA);
    return (ERRO);
  }
  while (1) {
    if (fgets (bufaux, TAMFIL, fd) == NULL) {
      fclose (fd);
      return (ERRO);
    }
    if (strncmp (nreq, bufaux, strlen (nreq))!=OK)
      continue;
    fseek (fd, (long) strlen (bufaux)*-1, 1);
    for (i=1; i<256; i++)
      if (buffer [i]==LF)
        buffer [i]=BRANCO;
    buffer [256]=LF;
    buffer [257]=NULO;
    fputs (buffer, fd);
    fclose (fd);
    return (OK);
  }
}

pega_prox_req_ativa (fdfila, buffer) 
FILE *fdfila;
char *buffer;
{
  char estado [60];

  while ((buffer=fgets (buffer, TAMFIL, fdfila)) != NULL) { /* pegar o 1o  */ 
    pega_cpo (buffer, FILSTA, estado);                      /* reg ativo.  */
    if (!strcmp (estado, DELETADO))
      continue;
    else
      return (OK);
   }
   if (buffer == NULL) 
     return (ERRO);
}

/* Manipulacao de Arquivos */

abre (fd, arq, modo)
int *fd;
char *arq, *modo;
{
  int oflag=0;
  char cmd [128];
   
  if (!strcmp (modo, "r"))
    oflag=O_RDONLY;
  else
    if (!strcmp (modo, "w"))
      oflag=O_WRONLY;
    else
      if (!strcmp (modo, "r+w"))
        oflag=O_RDWR;
      else
        if (!strcmp (modo, "a"))
          oflag=O_APPEND|O_WRONLY;
        else
          if (!strcmp (modo, "create")) {
            if ((*fd=creat (arq, 0777)) == ERRO) {
              printf ("Erro: nao foi possivel abrir arquivo %s", arq);
              exit (0);
            }
            return (OK);    
          }
  if ((*fd=open (arq, oflag)) == ERRO) {
    printf ("Erro na abertura do arquivo %s\n", arq);
    exit (0);
  }
}
 
le_seq (fd, buf, tamreg)
int fd, tamreg;
char *buf;
{
  int nlido;

  if ((nlido=read (fd, buf, tamreg)) == tamreg) 
    return (OK);
  return (ERRO);
}

pesq_reg (fd, buf, tamreg, chave)
int fd, tamreg;
char *buf, *chave;
{
  int nlido;
  while ((nlido=read (fd, buf, tamreg)) == tamreg) 
    if (strncmp (chave, buf, strlen (chave)) == OK)
      return (OK);
  return (ERRO);
}

grava_reg (fd, buf, tamreg)
int fd, tamreg;
char *buf;
{
  char *ptr;
  int flag=0;

  for (ptr=buf; ptr < (buf+tamreg); ptr++) {
    if ((*ptr == LF) || (*ptr == NULO))
      flag=1;
    if (flag)
      *ptr=BRANCO;
  }
  ptr=buf+tamreg-1;
  *ptr=LF;
  write (fd, buf, tamreg);
}

regrava_reg (fd, buf, tamreg)
int fd, tamreg;
char *buf;
{
  char *ptr;
  int flag=0; 

  for (ptr=buf; ptr < (buf+tamreg); ptr++) {
    if ((*ptr == LF) || (*ptr == NULO))
      flag=1;
    if (flag)
      *ptr=BRANCO;
  }
  ptr=buf+tamreg-1;
  *ptr=LF;
  lseek (fd, (long)(tamreg*-1), 1);
  write (fd, buf, tamreg);
}

pega_linha_cmd (orig, ncampo, dest)
char *orig, *dest;
int ncampo;
{
  char *strtoken (), bufaux[512], *ptr;
  int i;
      
   strcpy (bufaux, orig);
   ptr=strtoken (bufaux, ESPACO);
   for (i=1; i < ncampo; i++)
     if ((ptr=strtoken (NULO, ESPACO)) == NULL)
       return (ERRO);
   strcpy (dest, ptr);
   return (OK);
}

pega_cpo (orig, ncampo, dest)
char *orig, *dest;
int ncampo;
{
  char *strtoken (), bufaux[512], *ptr;
  int i;
      
   strcpy (bufaux, orig);
   ptr=strtoken (bufaux, TOKEN);
   for (i=1; i < ncampo; i++)
     if ((ptr=strtoken (NULO, TOKEN)) == NULL)
       return (ERRO);
   strcpy (dest, ptr);
   return (OK);
}

poe_cpo (dest, ncampo, orig)
char *dest, *orig;
int ncampo;
{
  char bufaux [512], *ptr, *strtoken ();
  int i, tam;
    
  tam=strlen (dest);
  strcpy (bufaux, "");
  ptr=strtoken (dest, TOKEN);
  for (i=1;; i++) {
    if (i == ncampo) 
      strcat (bufaux, orig);
    else
      strcat (bufaux, ptr);
    if ((ptr=strtoken (NULO, TOKEN)) == NULL) {
      break;
    }
    strcat (bufaux, TOKEN);
  }
  strncpy (dest, bufaux, tam-1);
}

pegasn()
{
  char str[30];

  gets (str);
  if ((strcmp (str, "S") == 0) || (strcmp (str, "s") == 0)) 
    return (SIM);
  if ((strcmp (str, "N") == 0) || (strcmp (str, "n") == 0)) 
    return (NAO);
  return (INVALIDO);
}

char *strtoken (buf, tok)
char *buf, *tok;
{
  static char *ptri, *ptrf;

  if (buf != NULL) 
    ptri=ptrf=buf;
  else
    ptri=ptrf;
  for (; *ptrf != NULO; ptrf++) 
    if (*ptrf == *tok) {
      *ptrf++=NULO;
      return (ptri);
    }
  if (ptri == ptrf)
    return (NULL);
  else
    return (ptri); 
}

